#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

typedef struct {
	uint8_t r;
	uint8_t g;
	uint8_t b;
} rgb888;


int main (int argc, char *argv[])
{
	if (argc != 2) {
		printf("Please provide a single 24 bit color of the format `0xRRGGBB` as input.\n");
		return 1;
	}

	uint32_t in_888 = strtol(argv[1], NULL, 16);

	printf(" Input, 24 bit color, 6 char hex: #%06X\n\n", in_888);
	
	rgb888 split;
	split.r = (in_888 & 0x00FF0000) >> 16;
	split.g = (in_888 & 0x0000FF00) >> 8;
	split.b = in_888 & 0x000000FF;

	// Basic rounding
	rgb888 rounded = split;
	if (split.r & 4) rounded.r |= 8;
	if (split.g & 4) rounded.g |= 8;
	if (split.b & 4) rounded.b |= 8;

	int16_t rgb555 = ((split.b & 0xF8) << 7) | ((split.g & 0xF8) << 2) | (split.r >> 3);
	int16_t rgb555_rounded = ((rounded.b & 0xF8) << 7) | ((rounded.g & 0xF8) << 2) | (rounded.r >> 3);

	if (rgb555 != rgb555_rounded) {
		printf("Output, 15 bit color, 6 char hex, unrounded: #%06X\n", (uint32_t) ((split.r & 0xF8) << 16) | ((split.g & 0xF8) << 8) | (split.b & 0xF8));
		printf("Output, 15 bit color, 6 char hex, rounded:   #%06X\n", (uint32_t) ((rounded.r & 0xF8) << 16) | ((rounded.g & 0xF8) << 8) | (rounded.b & 0xF8));

		printf("Output, 15 bit color, 4 char hex, unrounded:  0x%04X\n", rgb555);
		printf("Output, 15 bit color, 4 char hex, rounded:    0x%04X\n", rgb555_rounded);
	} else {
		printf("Output, 15 bit color, 6 char hex:   #%06X\n", (uint32_t) ((split.r & 0xF8) << 16) | ((split.g & 0xF8) << 8) | (split.b & 0xF8));
		printf("Output, 15 bit color, 4 char hex:  0x%04X\n", rgb555);
	}
	return 0;
}
